/*
 * Copyright (c) 2016 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "launcher.h"

extern "C"
{
  typedef int (*coreclr_initialize_ptr)(
      const char* exePath,
      const char* appDomainFriendlyName,
      int propertyCount,
      const char** propertyKeys,
      const char** propertyValues,
      void** hostHandle,
      unsigned int* domainId);

  typedef int (*coreclr_execute_assembly_ptr)(
      void* hostHandle,
      unsigned int domainId,
      int argc,
      const char** argv,
      const char* managedAssemblyPath,
      unsigned int* exitCode);

  typedef int (*coreclr_shutdown_ptr)(
      void* hostHandle,
      unsigned int domainId);

  typedef int (*coreclr_create_delegate_ptr)(
      void* hostHandle,
      unsigned int domainId,
      const char* entryPointAssemblyName,
      const char* entryPointTypeName,
      const char* entryPointMethodName,
      void** delegate);
}

namespace tizen {
namespace runtime {
namespace dotnetcore {

typedef void (*PreparedFunctionPtr)();
typedef bool (*LaunchFunctionPtr)(const char* root, const char* path, int argc, char* argv[]);

class CoreRuntime : public tizen::runtime::LauncherInterface
{
  public:
    CoreRuntime();
    ~CoreRuntime();
    int Initialize(bool standalone) override;
    void Dispose() override;
    int RunManagedLauncher(const char* app_id, const char* app_base, const char* tpa_list) override;
    int Launch(const char* app_id, const char* root, const char* path, int argc, char* argv[]) override;

  private:
    bool InitializeCoreClr(const char* app_id, const char* assembly_probe_paths, const char* pinvoke_probe_paths, const char* tpa);
    coreclr_initialize_ptr InitializeClr;
    coreclr_execute_assembly_ptr ExecuteAssembly;
    coreclr_shutdown_ptr Shutdown;
    coreclr_create_delegate_ptr CreateDelegate;
    std::string DeviceAPIDirectory;
    std::string RuntimeDirectory;
    std::string LauncherAssembly;
    std::string NativeLibDirectory;
    void* coreclrLib;
    void* hostHandle;
    unsigned int domainId;
    PreparedFunctionPtr PreparedFunction;
    LaunchFunctionPtr LaunchFunction;
};

}  // dotnetcore
}  // namespace runtime
}  // namespace tizen
