/*
 * Copyright (c) 2016 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

using System;
using System.IO;
using System.Runtime.InteropServices;
using System.Runtime.CompilerServices;

namespace Tizen.Runtime
{
#if CLOG
    internal static class Log
    {
        static void Print(string tag, string message)
        {
            string[] lines = message.Split('\r');
            foreach (string line in lines)
            {
                Console.WriteLine($"{tag} : {message}");
            }
        }

        public static void Debug(string tag, string message)
        {
            Print($"D/{tag}", message);
        }

        public static void Info(string tag, string message)
        {
            Print($"I/{tag}", message);
        }

        public static void Error(string tag, string message)
        {
            Print($"E/{tag}", message);
        }
    }
#endif
    internal static class ALog
    {
        const string Library = "libdlog.so.0";
        const string TAG = "DOTNET_LAUNCHER";

        public static void Debug(string message,
                [CallerFilePath] string file = "",
                [CallerMemberName] string func = "",
                [CallerLineNumber] int line = 0)
        {
            Print(LogPriority.DLOG_DEBUG, TAG, message, file, func, line);
        }

        public static void Info(string message,
                [CallerFilePath] string file = "",
                [CallerMemberName] string func = "",
                [CallerLineNumber] int line = 0)
        {
            Print(LogPriority.DLOG_DEBUG, TAG, message, file, func, line);
        }

        public static void Error(string message,
                [CallerFilePath] string file = "",
                [CallerMemberName] string func = "",
                [CallerLineNumber] int line = 0)
        {
            Print(LogPriority.DLOG_DEBUG, TAG, message, file, func, line);
        }

        internal enum LogPriority
        {
            DLOG_UNKNOWN = 0,
            DLOG_DEFAULT,
            DLOG_VERBOSE,
            DLOG_DEBUG,
            DLOG_INFO,
            DLOG_WARN,
            DLOG_ERROR,
            DLOG_FATAL,
            DLOG_SILENT,
            DLOG_PRIO_MAX,
        }

        private static void Print(LogPriority priority, string tag, string message, string file, string func, int line)
        {
            FileInfo finfo = new FileInfo(file);
            Print(priority, tag, "%s: %s(%d) > %s", finfo.Name, func, line, message); 
        }

        [DllImportAttribute(Library, EntryPoint = "dlog_print")]
        internal static extern int Print(LogPriority prio, string tag, string fmt, string file, string func, int line, string msg);
    }
}
